<?php
declare(strict_types=1);

namespace Catequil\Db;

use Catequil\Db\Exception\UnsupportedDriverException;
use Doctrine\DBAL\Connection as BaseConnection;
use Doctrine\DBAL\Driver\PDO\MySQL\Driver as MysqlDriver;
use function array_walk;
use function count;
use function preg_replace;

/**
 * Class Connection
 *
 * @package Catequil\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class Connection extends BaseConnection
{
    use Feature\MultiInsert;
    use Feature\MultiUpsert;
    use Feature\Upsert;

    /**
     * @return void
     * @throws UnsupportedDriverException
     */
    private function assertMysql(): void
    {
        if (!$this->getDriver() instanceof MysqlDriver) {
            throw new UnsupportedDriverException();
        }
    }

    /**
     * @param string $identifier
     * @return string
     */
    public function escapeMySqlIdentifier(string $identifier): string
    {
        return '`' . preg_replace('/\W+/', '', $identifier) . '`';
    }

    protected function generateMultiColumns(array $rows, array $columns): array
    {
        return \array_map(
            fn (string $column) => $this->escapeMySqlIdentifier($column),
            empty($columns) ? \array_keys(\current($rows)) : $columns
        );
    }

    /**
     * @param array $rows
     *
     * @return array
     */
    public function generateMultiParams(array $rows): array
    {
        $params = [];

        foreach ($rows as $row) {
            foreach ($row as $value) {
                $params[] = $value;
            }
        }

        return $params;
    }

    /**
     * @param array $rows
     * @return array
     */
    private function flattenRows(array $rows): array
    {
        $flattened = [];

        array_walk($rows, function (array $row) use (&$flattened) {
            foreach ($row as $col) {
                $flattened[] = $col;
            }
        });

        return $flattened;
    }
}
