<?php
declare(strict_types=1);

namespace Catequil\Db\Feature;

use Doctrine\DBAL\Exception as DBALException;

use function array_fill;
use function count;
use function implode;
use function intval;
use function sprintf;

/**
 * Trait MultiInsert
 *
 * @package Catequil\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
trait MultiInsert
{
    /**
     * executes "INSERT" sql statement by multi array of values
     * to be used for bulk inserts
     *
     * @param string $table
     * @param array $rows
     * @param array $columns
     *
     * @return int
     *
     * @throws DBALException
     */
    public function multiInsert(string $table, array $rows, array $columns = []): int
    {
        $this->assertMysql();

        $sql = $this->generateMultiInsertSql($table, $rows, $columns);
        $params = $this->generateMultiParams($rows);

        return intval($this->executeStatement($sql, $params));
    }

    /**
     * @param string $table
     * @param array $rows
     * @param array $columns
     * @return string
     */
    public function generateMultiInsertSql(string $table, array $rows, array $columns = []): string
    {
        $columns = $this->generateMultiColumns($rows, $columns);

        return sprintf(
            'INSERT INTO `%s` (%s) VALUES %s',
            $table,
            implode(', ', $columns),
            implode(', ', array_fill(
                0,
                count($rows),
                '(' . implode(', ', array_fill(0, count($columns), '?')) . ')'
            )),
        );
    }
}
