<?php
declare(strict_types=1);

namespace Catequil\Db\Feature;

use Doctrine\DBAL\Exception as DBALException;

use function array_filter;
use function array_keys;
use function array_map;
use function current;
use function in_array;
use function sprintf;

/**
 * Trait MultiUpsert
 *
 * @package Catequil\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
trait MultiUpsert
{
    /**
     * @param string $table
     * @param array $rows
     * @param array $ignoreForUpdate
     *
     * @return int
     * @throws DBALException
     */
    public function multiUpsert(string $table, array $rows, array $ignoreForUpdate = []): int
    {
        $this->assertMysql();

        $sql = $this->generateMultiUpsertSql($table, $rows, $ignoreForUpdate);
        return $this->executeStatement($sql, $this->generateMultiParams($rows));
    }

    public function generateMultiUpsertSql(string $table, array $rows, array $ignoreForUpdate = []): string
    {
        $cols = array_map(
            [$this, 'escapeMySqlIdentifier'],
            array_filter(
                array_keys(current($rows)),
                fn (string $col) => !in_array($col, $ignoreForUpdate),
            )
        );

        return sprintf(
            '%s ON DUPLICATE KEY UPDATE %s',
            $this->generateMultiInsertSql($table, $rows),
            implode(', ', array_filter(
                array_map(
                    fn (string $col) => sprintf('%1$s=VALUES(%1$s)', $col),
                    $cols
                )
            ))
        );
    }
}
