<?php
declare(strict_types=1);

namespace Catequil\Db\Feature;

use Doctrine\DBAL\Exception as DBALException;

use function array_filter;
use function array_keys;
use function array_map;
use function array_values;
use function implode;
use function in_array;
use function sprintf;

/**
 * Trait Upsert
 *
 * @package Catequil\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
trait Upsert
{
    /**
     * @param string $table
     * @param array $data
     * @param array $ignoreForUpdate
     * @return int
     * @throws DBALException
     */
    public function upsert(string $table, array $data, array $ignoreForUpdate = []): int
    {
        $this->assertMysql();

        $sql = $this->generateUpsertSql($table, $data, $ignoreForUpdate);
        return $this->executeStatement($sql, array_values($data));
    }

    /**
     * @param string $table
     * @param array $data
     * @param array $ignoreForUpdate
     * @return string
     */
    public function generateUpsertSql(string $table, array $data, array $ignoreForUpdate = []): string
    {
        $cols = array_map(
            [$this, 'escapeMySqlIdentifier'],
            array_filter(
                array_keys($data),
                fn (string $col) => !in_array($col, $ignoreForUpdate)
            )
        );

        return sprintf(
            '%s ON DUPLICATE KEY UPDATE %s, id=LAST_INSERT_ID(id)',
            $this->generateMultiInsertSql($table, [$data]),
            implode(', ', array_filter(
                array_map(
                    fn (string $col) => sprintf('%1$s=VALUES(%1$s)', $col),
                    $cols
                )
            ))
        );
    }
}
