<?php
declare(strict_types=1);

namespace CatequilTest\Db;

use Ak\Test\AbstractTestCase;
use Catequil\Db\ConfigProvider;
use Catequil\Db\Connection;
use Laminas\ConfigAggregator\ArrayProvider;

/**
 * Class AbstractDbTestCase
 *
 * @package CatequilTest\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
abstract class AbstractDbTestCase extends AbstractTestCase
{
    protected array $providers = [
        ConfigProvider::class,
    ];

    public function __construct(?string $name = null, array $data = [], $dataName = '')
    {
        parent::__construct($name, $data, $dataName);

        $this->providers[] = new ArrayProvider([
            'db' => [
                'connections' => [
                    'db.default' => [
                        'user' => 'root',
                        'password' => 'password',
                        'dbname' => 'test',
                    ],
                    'db.memory' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);
    }

    /**
     * @param string $name
     * @return Connection
     * @noinspection PhpDocMissingThrowsInspection
     */
    protected function getConnection(string $name = 'db.default'): Connection
    {
        $container = $this->getContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        return $container->get($name);
    }
}
