<?php
declare(strict_types=1);

namespace CatequilTest\Db\Container;

use Ak\Test\AbstractTestCase;
use Catequil\Db\ConfigProvider;
use Catequil\Db\Connection;
use Catequil\Db\Container\ConnectionAbstractFactory;
use Doctrine\DBAL\Driver\PDO\SQLite\Driver;
use Laminas\ServiceManager\Exception\ServiceNotFoundException;

/**
 * Class ConnectionAbstractFactoryTest
 *
 * @package CatequilTest\Db\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConnectionAbstractFactoryTest extends AbstractTestCase
{
    protected array $providers = [
        ConfigProvider::class,
    ];

    private ConnectionAbstractFactory $factory;

    protected function setUp(): void
    {
        $this->factory = new ConnectionAbstractFactory();
    }

    public function testCanCreateValidConnections()
    {
        $container = $this->getContainer();

        $this->assertTrue($this->factory->canCreate($container, 'db'));
        $this->assertTrue($this->factory->canCreate($container, 'db.default'));
        $this->assertFalse($this->factory->canCreate($container, 'db.invalid'));
    }

    public function testWillThrowExceptionForInvalidConnection()
    {
        $container = $this->getContainer();

        $this->expectException(ServiceNotFoundException::class);
        ($this->factory)($container, 'invalid');
    }

    public function testCanCreateDefaultConnection()
    {
        $container = $this->getContainer([
            'db' => [
                'connections' => [
                    'db.default' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);

        $conn = ($this->factory)($container, 'db');

        $this->assertInstanceOf(Connection::class, $conn);
        $this->assertInstanceOf(Driver::class, $conn->getDriver());
    }

    public function testCanCreateNamedConnections()
    {
        $container = $this->getContainer([
            'db' => [
                'connections' => [
                    'some-connection' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);

        $conn = ($this->factory)($container, 'some-connection');

        $this->assertInstanceOf(Connection::class, $conn);
        $this->assertInstanceOf(Driver::class, $conn->getDriver());
    }
}
