<?php
declare(strict_types=1);

namespace CatequilTest\Db\Feature;

use Catequil\Db\Exception\UnsupportedDriverException;
use CatequilTest\Db\AbstractDbTestCase;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\DBAL\Schema\Comparator;
use Doctrine\DBAL\Schema\SchemaException;

/**
 * Class FeatureTestCase
 *
 * @package CatequilTest\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
abstract class FeatureTestCase extends AbstractDbTestCase
{
    /**
     * @throws SchemaException
     * @throws DBALException
     */
    protected function setUp(): void
    {
        parent::setUp();

        $conn = $this->getConnection();
        $sm = $conn->createSchemaManager();

        if (!$sm->tablesExist('catequil_db')) {
            $fromSchema = $sm->createSchema();
            $toSchema = clone $fromSchema;

            $table = $toSchema->createTable('catequil_db');
            $table->addColumn('id', 'bigint', ['unsigned' => true]);
            $table->addColumn('value', 'string', ['length' => 32]);
            $table->setPrimaryKey(['id']);

            $comparator = new Comparator();
            $schemaDiff = $comparator->compareSchemas($fromSchema, $toSchema);

            $queries = $schemaDiff->toSql($conn->getDatabasePlatform());

            foreach ($queries as $query) {
                $conn->executeQuery($query);
            }
        }

        $conn->executeQuery('TRUNCATE catequil_db');
    }

    /**
     * @param int $count
     * @param bool $unique
     * @param bool $associative
     *
     * @return array
     */
    protected function generateRandomRows(int $count, bool $unique = false, bool $associative = false): array
    {
        $values = [];

        while (\count($values) < $count) {
            $value = \md5((string) \rand());

            if ($unique && \in_array($value, $values)) {
                continue;
            }

            $values[] = $value;
        }

        return \array_map(
            fn (int $i, string $value) => $associative
                ? ['id' => $i+1, 'value' => $value]
                : [$i+1, $value],
            \array_keys($values),
            \array_values($values)
        );
    }
}
