<?php
declare(strict_types=1);

namespace CatequilTest\Db\Feature;

use Catequil\Db\Exception\UnsupportedDriverException;

/**
 * Class MultiInsertTest
 *
 * @package CatequilTest\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class MultiInsertTest extends FeatureTestCase
{
    /**
     * @dataProvider dataProvider
     */
    public function testSqlQuery(array $data, string $sql, array $params)
    {
        $conn = $this->getConnection();

        self::assertSame($sql, $conn->generateMultiInsertSql('test', $data));
        self::assertSame($params, $conn->generateMultiParams($data));
    }

    public function dataProvider(): array
    {
        return [[
            'data' => [['a' => 'av']],
            'sql' => 'INSERT INTO `test` (`a`) VALUES (?)',
            'params' => ['av'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => 'bv',
            ], [
                'a' => 'av',
                'b' => 'bv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`) VALUES (?, ?), (?, ?)',
            'params' => ['av', 'bv', 'av', 'bv'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => '0',
                'c' => 'cv',
                'd' => 'dv',
            ], [
                'a' => 'av',
                'b' => 'bv',
                'c' => '',
                'd' => 'dv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?), (?, ?, ?, ?)',
            'params' => ['av', '0', 'cv', 'dv', 'av', 'bv', '', 'dv'],
        ], [
            'data' => [[
                'user' => 'vuser',
                'key' => 'vkey',
                'order' => 'vorder',
                'group' => 'vgroup',
            ]],
            'sql' => 'INSERT INTO `test` (`user`, `key`, `order`, `group`) VALUES (?, ?, ?, ?)',
            'params' => ['vuser', 'vkey', 'vorder', 'vgroup'],
        ]];
    }
}
