<?php
declare(strict_types=1);

namespace CatequilTest\Db\Feature;

use function array_values;

/**
 * Class UpsertTest
 *
 * @package CatequilTest\Db\Feature
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class UpsertTest extends FeatureTestCase
{
    /**
     * @dataProvider dataProvider
     */
    public function testSqlQuery(array $data, string $sql, array $params)
    {
        $conn = $this->getConnection();

        self::assertSame($sql, $conn->generateUpsertSql('test', $data));
        self::assertSame($params, array_values($data));
    }

    public function testIgnoresColumns()
    {
        $data = [
            'a' => 'av',
            'b' => '0',
            'c' => 'cv',
            'd' => 'dv',
        ];
        $sql = 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `d`=VALUES(`d`), id=LAST_INSERT_ID(id)';

        $conn = $this->getConnection();

        self::assertSame($sql, $conn->generateUpsertSql('test', $data, ['b', 'c']));
    }

    public function dataProvider(): array
    {
        return [[
            'data' => ['a' => 'av'],
            'sql' => 'INSERT INTO `test` (`a`) VALUES (?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), id=LAST_INSERT_ID(id)',
            'params' => ['av'],
        ], [
            'data' => [
                'a' => 'av',
                'b' => 'bv',
            ],
            'sql' => 'INSERT INTO `test` (`a`, `b`) VALUES (?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`), id=LAST_INSERT_ID(id)',
            'params' => ['av', 'bv'],
        ], [
            'data' => [
                'a' => 'av',
                'b' => '0',
                'c' => '',
                'd' => 'dv',
            ],
            'sql' => 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`), `c`=VALUES(`c`), `d`=VALUES(`d`), id=LAST_INSERT_ID(id)',
            'params' => ['av', '0', '', 'dv'],
        ], [
            'data' => [
                'user' => 'vuser',
                'key' => 'vkey',
                'order' => 'vorder',
                'group' => 'vgroup',
            ],
            'sql' => 'INSERT INTO `test` (`user`, `key`, `order`, `group`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `user`=VALUES(`user`), `key`=VALUES(`key`), `order`=VALUES(`order`), `group`=VALUES(`group`), id=LAST_INSERT_ID(id)',
            'params' => ['vuser', 'vkey', 'vorder', 'vgroup'],
        ]];
    }
}
